from unit.api.base_resource import BaseResource
from unit.models.fee import *
from unit.models.codecs import DtoDecoder


class FeeResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "fees"

    def create(self, request: CreateFeeRequest) -> Union[UnitResponse[FeeDTO], UnitError]:
        payload = request.to_json_api()
        response = super().post(self.resource, payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[FeeDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())




import json
from unit.models import *


class AtmLocationDTO(object):
    def __init__(self, network: int, location_name: str, coordinates: Coordinates, address: Address, distance: int,
                 surcharge_free: bool, accept_deposits: bool):
        self.type = "atmLocation"
        self.attributes = {"network": network, "locationName": location_name, "coordinates": coordinates,
                           "address": address, "distance": distance, "surchargeFree": surcharge_free,
                           "acceptDeposits": accept_deposits}

    @staticmethod
    def from_json_api(_type, attributes):
        return AtmLocationDTO(attributes["network"], attributes["locationName"],
                              Coordinates.from_json_api(attributes["coordinates"]),
                              Address.from_json_api(attributes["address"]), attributes["distance"],
                              attributes["surchargeFree"], attributes["acceptDeposits"])


class GetAtmLocationParams(object):
    def __init__(self, search_radius: Optional[int] = None, coordinates: Optional[Coordinates] = None,
                 postal_code: Optional[str] = None, address: Optional[Address] = None):
        self.search_radius = search_radius
        self.coordinates = coordinates
        self.postal_code = postal_code
        self.address = address




from unit.api.base_resource import BaseResource
from unit.models.authorization_request import *
from unit.models.codecs import DtoDecoder


class AuthorizationRequestResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "authorization-requests"

    def get(self, authorization_id: str) -> Union[UnitResponse[PurchaseAuthorizationRequestDTO], UnitError]:
        response = super().get(f"{self.resource}/{authorization_id}")
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[PurchaseAuthorizationRequestDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def list(self, params: PurchaseAuthorizationRequestListParams = PurchaseAuthorizationRequestListParams()) \
            -> Union[UnitResponse[List[PurchaseAuthorizationRequestDTO]], UnitError]:
        parameters = {"page[limit]": params.limit, "page[offset]": params.offset}

        if params.account_id != "":
            parameters |= {"filter[accountId]": params.account_id}

        if params.customer_id != "":
            parameters |= {"filter[customerId]": params.customer_id}

        response = super().get(self.resource, parameters)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[PurchaseAuthorizationRequestDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def approve(self, request: ApproveAuthorizationRequest) -> Union[UnitResponse[PurchaseAuthorizationRequestDTO], UnitError]:
        payload = request.to_json_api()
        response = super().post(f"{self.resource}/{request.authorization_id}/approve", payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[PurchaseAuthorizationRequestDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def decline(self, request: DeclineAuthorizationRequest) -> Union[UnitResponse[PurchaseAuthorizationRequestDTO], UnitError]:
        payload = request.to_json_api()
        response = super().post(f"{self.resource}/{request.authorization_id}/decline", payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[PurchaseAuthorizationRequestDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())




import json
from datetime import datetime, date
from typing import Literal, Optional
from unit.utils import date_utils
from unit.models import *

class BaseEvent(object):
    def __init__(self, id: str, created_at: datetime, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        self.id = id
        self.attributes = {"createdAt": created_at, "tags": tags}
        self.relationships = relationships


class AccountClosedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, close_reason: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'account.closed'
        self.attributes["closeReason"] = close_reason

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AccountClosedEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["closeReason"],
                                  attributes.get("tags"), relationships)


class AccountFrozenEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, freeze_reason: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'account.frozen'
        self.attributes["freezeReason"] = freeze_reason

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AccountFrozenEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["freezeReason"],
                                  attributes.get("tags"), relationships)


class ApplicationDeniedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'application.denied'

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return ApplicationDeniedEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes.get("tags"),
                                      relationships)


class ApplicationPendingReviewEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'application.pendingReview'

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return ApplicationPendingReviewEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                             attributes.get("tags"), relationships)


class ApplicationAwaitingDocumentsEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'application.awaitingDocuments'

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return ApplicationAwaitingDocumentsEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                                 attributes.get("tags"), relationships)


class AuthorizationCreatedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, card_last_4_digits: str, recurring: str,
                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'authorization.created'
        self.attributes["cardLast4Digits"] = card_last_4_digits
        self.attributes["recurring"] = recurring

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AuthorizationCreatedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                         attributes["cardLast4Digits"], attributes["recurring"],
                                         attributes.get("tags"), relationships)

class AuthorizationRequestApprovedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, amount: str, status: str, approved_amount: str,
                 partial_approval_allowed: str, merchant: Dict[str, str], recurring: str,
                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'authorizationRequest.approved'
        self.attributes["amount"] = amount
        self.attributes["status"] = status
        self.attributes["approvedAmount"] = approved_amount
        self.attributes["partialApprovalAllowed"] = partial_approval_allowed
        self.attributes["merchant"] = merchant
        self.attributes["recurring"] = recurring


    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AuthorizationRequestApprovedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                                 attributes["amount"], attributes["status"],
                                                 attributes["approvedAmount"], attributes["partialApprovalAllowed"],
                                                 attributes["merchant"], attributes["recurring"],
                                                 attributes.get("tags"), relationships)


class AuthorizationRequestDeclinedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, amount: str, status: str, decline_reason: str,
                 partial_approval_allowed: str, merchant: Dict[str, str], recurring: str,
                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'authorizationRequest.declined'
        self.attributes["amount"] = amount
        self.attributes["status"] = status
        self.attributes["declineReason"] = decline_reason
        self.attributes["partialApprovalAllowed"] = partial_approval_allowed
        self.attributes["merchant"] = merchant
        self.attributes["recurring"] = recurring


    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AuthorizationRequestDeclinedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                                 attributes["amount"], attributes["status"],
                                                 attributes["declineReason"], attributes["partialApprovalAllowed"],
                                                 attributes["merchant"], attributes["recurring"],
                                                 attributes.get("tags"), relationships)


class AuthorizationRequestPendingEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, amount: str, status: str, partial_approval_allowed: str,
                 merchant: Dict[str, str], recurring: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'authorizationRequest.pending'
        self.attributes["amount"] = amount
        self.attributes["status"] = status
        self.attributes["approvedAmount"] = approved_amount
        self.attributes["partialApprovalAllowed"] = partial_approval_allowed
        self.attributes["merchant"] = merchant
        self.attributes["recurring"] = recurring


    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AuthorizationRequestPendingEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                                attributes["amount"], attributes["status"],
                                                attributes["approvedAmount"], attributes["partialApprovalAllowed"],
                                                attributes["merchant"], attributes["recurring"], attributes.get("tags"),
                                                relationships)

class CardActivatedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'card.activated'

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CardActivatedEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes.get("tags"),
                                  relationships)


class CardStatusChangedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, new_status: str, previous_status: str,
                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'card.statusChanged'
        self.attributes["newStatus"] = new_status
        self.attributes["previousStatus"] = previous_status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CardStatusChangedEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["newStatus"],
                                      attributes["previousStatus"], attributes.get("tags"), relationships)


class CheckDepositCreatedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, status: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'checkDeposit.created'
        self.attributes["status"] = status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CheckDepositCreatedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                      attributes["status"], attributes.get("tags"), relationships)

class CheckDepositClearingEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, previous_status: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'checkDeposit.clearing'
        self.attributes["previousStatus"] = previous_status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CheckDepositClearingEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                      attributes["previousStatus"], attributes.get("tags"), relationships)


class CheckDepositSentEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, previous_status: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'checkDeposit.sent'
        self.attributes["previousStatus"] = previous_status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CheckDepositSentEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                      attributes["previousStatus"], attributes.get("tags"), relationships)


class CheckDepositReturnedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, previous_status: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'checkDeposit.returned'
        self.attributes["previousStatus"] = previous_status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CheckDepositReturnedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                      attributes["previousStatus"], attributes.get("tags"), relationships)


class CustomerCreatedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'customer.created'

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CustomerCreatedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                    attributes.get("tags"), relationships)

class DocumentApprovedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'document.approved'

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return DocumentApprovedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                    attributes.get("tags"), relationships)

class DocumentRejectedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, reason: str, reason_code: str,
                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'document.rejected'
        self.attributes["reason"] = reason
        self.attributes["reasonCode"] = reason_code

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return DocumentRejectedEvent(_id, date_utils.to_datetime(attributes["createdAt"]),
                                     attributes["reason"], attributes["reasonCode"], attributes.get("tags"),
                                     relationships)


class PaymentClearingEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, previous_status: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'payment.clearing'
        self.attributes["previousStatus"] = previous_status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return PaymentClearingEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["previousStatus"],
                                    attributes.get("tags"), relationships)

class PaymentSentEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, previous_status: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'payment.sent'
        self.attributes["previousStatus"] = previous_status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return PaymentSentEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["previousStatus"],
                                    attributes.get("tags"), relationships)

class PaymentReturnedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, previous_status: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'payment.returned'
        self.attributes["previousStatus"] = previous_status

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return PaymentReturnedEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["previousStatus"],
                                    attributes.get("tags"), relationships)

class StatementsCreatedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, period: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'statements.created'
        self.attributes["period"] = period

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return StatementsCreatedEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["period"],
                                    attributes.get("tags"), relationships)

class TransactionCreatedEvent(BaseEvent):
    def __init__(self, id: str, created_at: datetime, summary: str, direction: str, amount: str,
                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        BaseEvent.__init__(self, id, created_at, tags, relationships)
        self.type = 'transaction.created'
        self.attributes["summary"] = summary
        self.attributes["direction"] = direction
        self.attributes["amount"] = amount

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return TransactionCreatedEvent(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["summary"],
                                       attributes["direction"], attributes["amount"], attributes.get("tags"),
                                       relationships)

EventDTO = Union[AccountClosedEvent, AccountFrozenEvent, ApplicationDeniedEvent, ApplicationAwaitingDocumentsEvent,
                 ApplicationPendingReviewEvent, CardActivatedEvent, CardStatusChangedEvent,
                 AuthorizationCreatedEvent, AuthorizationRequestDeclinedEvent, AuthorizationRequestPendingEvent,
                 AuthorizationRequestApprovedEvent, DocumentApprovedEvent, DocumentRejectedEvent,
                 CheckDepositCreatedEvent, CheckDepositClearingEvent, CheckDepositSentEvent,
                 CheckDepositReturnedEvent, CustomerCreatedEvent, PaymentClearingEvent, PaymentSentEvent,
                 PaymentReturnedEvent, StatementsCreatedEvent, TransactionCreatedEvent]




from unit.api.base_resource import BaseResource
from unit.models.transaction import ReturnedReceivedAchTransactionDTO
from unit.models.returnAch import *
from unit.models.codecs import DtoDecoder


class ReturnAchResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "returns"

    def return_ach(self, request: ReturnReceivedAchTransactionRequest) -> Union[UnitResponse[ReturnedReceivedAchTransactionDTO], UnitError]:
        payload = request.to_json_api()
        response = super().post(f"{self.resource}/{request.transaction_id}", payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[ReturnedReceivedAchTransactionDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())




import json
from unit.models import *


class StatementDTO(object):
    def __init__(self, id: str, _type: str, period: str, relationships: Optional[Dict[str, Relationship]]):
        self.id = id
        self.type = _type
        self.attributes = {"period": period}
        self.relationships = relationships

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return StatementDTO(_id, _type, attributes["period"], relationships)


OutputType = Literal["html", "pdf"]

class GetStatementParams(object):
    def __init__(self, statement_id: str, output_type: Optional[OutputType] = "html", language: Optional[str] = "en",
                 customer_id: Optional[str] = None):
        self.statement_id = statement_id
        self.output_type = output_type
        self.language = language
        self.customer_id = customer_id




import json
from typing import Optional
from unit.models import *


class InstitutionDTO(object):
    def __init__(self, routing_number: str, name: str, is_ach_supported: bool, is_wire_supported: bool,
                 address: Optional[Address] = None):
        self.type = "institution"
        self.attributes = {"routingNumber": routing_number, "name": name, "address": address,
                           "isACHSupported": is_ach_supported, "isWireSupported": is_wire_supported}

    def from_json_api(_id, _type, attributes, relationships):
        return InstitutionDTO(
            attributes["routingNumber"], attributes["name"], attributes["isACHSupported"],
            attributes["isWireSupported"], attributes.get("address"))




import json
from typing import Optional
from unit.models import *
from unit.utils import date_utils


class AuthorizationDTO(object):
    def __init__(self, id: str, created_at: datetime, amount: int, card_last_4_digits: str, merchant_name: str,
                 merchant_type: int, merchant_category: str, merchant_location: Optional[str], recurring: bool,
                 tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        self.id = id
        self.type = "authorization"
        self.attributes = {"createdAt": created_at, "amount": amount, "cardLast4Digits": card_last_4_digits,
                           "merchant": { "name": merchant_name, "type": merchant_type, "category": merchant_category,
                                         "location": merchant_location}, "recurring": recurring, "tags": tags}
        self.relationships = relationships

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AuthorizationDTO(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["amount"],
                                attributes["cardLast4Digits"], attributes["merchant"]["name"],
                                attributes["merchant"]["type"], attributes["merchant"]["category"],
                                attributes["merchant"].get("location"), attributes["recurring"],
                                attributes.get("tags"), relationships)


class AuthorizationListParams(object):
    def __init__(self, limit: int = 100, offset: int = 0, account_id: str = "", customer_id: str = "",
                 card_id: str = "", since: str = "", until: str = ""):
        self.limit = limit
        self.offset = offset
        self.account_id = account_id
        self.customer_id = customer_id
        self.card_id = card_id
        self.since = since
        self.until = until




from unit.api.base_resource import BaseResource
from unit.models.atm_location import *
from unit.models.codecs import DtoDecoder, UnitEncoder

class AtmLocationResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "atm-locations"

    def get(self, request: GetAtmLocationParams) -> Union[UnitResponse[List[AtmLocationDTO]], UnitError]:
        params = {}

        if request.coordinates:
            params["filter[coordinates]"] = json.dumps(request.coordinates, cls=UnitEncoder)

        if request.address:
            params["filter[address]"] = json.dumps(request.address, cls=UnitEncoder)

        if request.postal_code:
            params["filter[postalCode]"] = json.dumps(request.postal_code, cls=UnitEncoder)

        if request.search_radius:
            params["filter[searchRadius]"] = request.search_radius

        response = super().get(self.resource, params)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[AtmLocationDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())




from unit.api.base_resource import BaseResource
from unit.models.institution import *
from unit.models.codecs import DtoDecoder


class InstitutionResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "institutions"

    def get(self, routing_number: str) -> Union[UnitResponse[InstitutionDTO], UnitError]:
        response = super().get(f"{self.resource}/{routing_number}", None)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[InstitutionDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())



import json
from typing import Optional, Literal
from unit.models import *
from unit.utils import date_utils

PurchaseAuthorizationRequestStatus = Literal["Pending", "Approved", "Declined"]
DeclineReason = Literal["AccountClosed", "CardExceedsAmountLimit", "DoNotHonor", "InsufficientFunds", "InvalidMerchant",
                        "ReferToCardIssuer", "RestrictedCard", "Timeout", "TransactionNotPermittedToCardholder"]

class PurchaseAuthorizationRequestDTO(object):
    def __init__(self, id: str, created_at: datetime, amount: int, status: PurchaseAuthorizationRequestStatus,
                 partial_approval_allowed: str, approved_amount: Optional[int], decline_reason: Optional[DeclineReason],
                 merchant_name: str, merchant_type: int, merchant_category: str, merchant_location: Optional[str],
                 recurring: bool, tags: Optional[Dict[str, str]], relationships: Optional[Dict[str, Relationship]]):
        self.id = id
        self.type = "purchaseAuthorizationRequest"
        self.attributes = {"createdAt": created_at, "amount": amount, "status": status,
                           "partialApprovalAllowed": partial_approval_allowed, "approvedAmount": approved_amount,
                           "declineReason": decline_reason, "merchant": { "name": merchant_name, "type": merchant_type,
                                                                          "category": merchant_category,
                                                                          "location": merchant_location},
                           "recurring": recurring, "tags": tags}
        self.relationships = relationships

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return PurchaseAuthorizationRequestDTO(_id, date_utils.to_datetime(attributes["createdAt"]),
                                               attributes["amount"], attributes["status"],
                                               attributes.get("partialApprovalAllowed"),
                                               attributes.get("approvedAmount"), attributes.get("declineReason"),
                                               attributes["merchant"]["name"], attributes["merchant"]["type"],
                                               attributes["merchant"]["category"],
                                               attributes["merchant"].get("location"), attributes["recurring"],
                                               attributes.get("tags"), relationships)


class PurchaseAuthorizationRequestListParams(object):
    def __init__(self, limit: int = 100, offset: int = 0, account_id: str = "", customer_id: str = ""):
        self.limit = limit
        self.offset = offset
        self.account_id = account_id
        self.customer_id = customer_id


class ApproveAuthorizationRequest(object):
    def __init__(self, authorization_id: str, amount: Optional[int] = None, tags: Optional[Dict[str, str]] = None):
        self.authorization_id = authorization_id
        self.amount = amount
        self.tags = tags

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "approveAuthorizationRequest",
                "attributes": {}
            }
        }

        if self.amount:
            payload["data"]["attributes"]["amount"] = self.amount

        if self.tags:
            payload["data"]["attributes"]["tags"] = self.tags

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())


class DeclineAuthorizationRequest(object):
    def __init__(self, authorization_id: str, reason: DeclineReason):
        self.authorization_id = authorization_id
        self.reason = reason

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "declineAuthorizationRequest",
                "attributes": {
                    "reason": self.reason
                }
            }
        }

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())



from unit.models import *
from unit.utils import date_utils


class APITokenDTO(object):
    def __init__(self, id: str, created_at: datetime, description: str, expiration: datetime, token: Optional[str],
                 source_ip: Optional[str]):
        self.id = id
        self.type = "apiToken"
        self.attributes = {"createdAt": created_at, "description": description, "expiration": expiration,
                           "token": token, "sourceIp": source_ip}

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return APITokenDTO(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["description"],
                           date_utils.to_datetime(attributes["expiration"]), attributes.get("token"),
                           attributes.get("sourceIp"))


class CreateAPITokenRequest(object):
    def __init__(self, user_id: str, description: str, scope: str, expiration: datetime,
                 source_ip: Optional[str] = None):
        self.user_id = user_id
        self.description = description
        self.scope = scope
        self.expiration = expiration
        self.source_ip = source_ip

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "apiToken",
                "attributes": {
                    "description": self.description,
                    "scope": self.scope,
                    "expiration": self.expiration
                }
            }
        }

        if self.source_ip:
            payload["data"]["attributes"]["sourceIp"] = self.source_ip

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())




import json
from datetime import datetime, date
from typing import Optional
from unit.utils import date_utils
from unit.models import *


class CounterpartyDTO(object):
    def __init__(self, id: str, created_at: datetime, name: str, routing_number: str, bank: Optional[str],
                 account_number: str, account_type: str, type: str, permissions: str,
                 relationships: [Dict[str, Relationship]]):
        self.id = id
        self.type = "achCounterparty"
        self.attributes = {"createdAt": created_at, "name": name, "routingNumber": routing_number, "bank": bank,
                           "accountNumber": account_number, "accountType": account_type, "type": type,
                           "permissions": permissions}
        self.relationships = relationships

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return CounterpartyDTO(_id, date_utils.to_datetime(attributes["createdAt"]), attributes["name"],
                               attributes["routingNumber"], attributes.get("bank"), attributes["accountNumber"],
                               attributes["accountType"], attributes["type"], attributes["permissions"], relationships)


class CreateCounterpartyRequest(object):
    def __init__(self, name: str, routing_number: str, account_number: str, account_type: str, type: str,
                 relationships: [Dict[str, Relationship]]):
        self.name = name
        self.routing_number = routing_number
        self.account_number = account_number
        self.account_type = account_type
        self.type = type
        self.relationships = relationships

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "achCounterparty",
                "attributes": {
                    "name": self.name,
                    "routingNumber": self.routing_number,
                    "accountNumber": self.account_number,
                    "accountType": self.account_type,
                    "type": self.type
                },
                "relationships": self.relationships
            }
        }

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())


class CreateCounterpartyWithTokenRequest(object):
    def __init__(self, name: str, type: str, plaid_processor_token: str, verify_name: Optional[bool],
                 relationships: [Dict[str, Relationship]]):
        self.name = name
        self.type = type
        self.plaid_processor_token = plaid_processor_token
        self.verify_name = verify_name
        self.relationships = relationships

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "achCounterparty",
                "attributes": {
                    "name": self.name,
                    "type": self.type,
                    "plaidProcessorToken": self.plaid_processor_token
                },
                "relationships": self.relationships
            }
        }

        if self.verify_name:
            payload["data"]["attributes"]["verifyName"] = self.verify_name

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())


class PatchCounterpartyRequest(object):
    def __init__(self, counterparty_id: str, plaid_processor_token: str, verify_name: Optional[bool] = False):
        self.counterparty_id = counterparty_id
        self.plaid_processor_token = plaid_processor_token
        self.verify_name = verify_name

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "counterparty",
                "attributes": {
                    "plaidProcessorToken": self.plaid_processor_token
                }
            }
        }

        if self.verify_name:
            payload["data"]["attributes"]["verifyName"] = self.verify_name

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())




import json
from typing import Literal
from unit.models import *

AchReturnReason = Literal["Unauthorized"]


class ReturnReceivedAchTransactionRequest(object):
    def __init__(self, transaction_id: str, reason: AchReturnReason, relationships: [Dict[str, Relationship]]):
        self.transaction_id = transaction_id
        self.reason = reason
        self.relationships = relationships

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "returnAch",
                "attributes": {
                    "reason": self.reason
                },
                "relationships": self.relationships
            }
        }

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())




import json
from datetime import datetime, date
from unit.utils import date_utils
from unit.models import *
from typing import Literal

ContentType = Literal["Json", "JsonAPI"]
WebhookStatus = Literal["Enabled", "Disabled"]


class WebhookDTO(object):
    def __init__(self, id: str, created_at: datetime, label: str, url: str, status: WebhookStatus,
                 content_type: ContentType, token: str):
        self.id = id
        self.type = 'webhook'
        self.attributes = {"createdAt": created_at, "label": label, "url": url, "status": status,
                           "contentType": content_type, "token": token}

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return WebhookDTO(
            _id, date_utils.to_datetime(attributes["createdAt"]), attributes["label"], attributes["url"],
            attributes["status"], attributes["contentType"], attributes["token"])


class CreateWebhookRequest(object):
    def __init__(self, label: str, url: str, token: str, content_type: ContentType):
        self.label = label
        self.url = url
        self.token = token
        self.content_type = content_type

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "webhook",
                "attributes": {
                    "label": self.label,
                    "url": self.url,
                    "token": self.token,
                    "contentType": self.content_type
                }
            }
        }

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())


class PatchWebhookRequest(object):
    def __init__(self, webhook_id: str, label: Optional[str] = None, url: Optional[str] = None,
                 content_type: Optional[ContentType] = None, token: Optional[str] = None):
        self.webhook_id = webhook_id
        self.label = label
        self.url = url
        self.content_type = content_type
        self.token = token

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "webhook",
                "attributes": {}
            }
        }

        if self.label:
            payload["data"]["attributes"]["label"] = self.label

        if self.url:
            payload["data"]["attributes"]["url"] = self.url

        if self.content_type:
            payload["data"]["attributes"]["contentType"] = self.content_type

        if self.token:
            payload["data"]["attributes"]["token"] = self.token

        return payload




from unit.api.base_resource import BaseResource
from unit.models.bill_pay import *
from unit.models.codecs import DtoDecoder


class BillPayResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "payments/billpay/billers"

    def get(self, params: GetBillersParams) -> Union[UnitResponse[List[BillerDTO]], UnitError]:
        parameters = {"name": params.name}
        if params.page:
            parameters["page"] = params.page

        response = super().get(self.resource, parameters)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[BillerDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())




from unit.api.base_resource import BaseResource
from unit.models.statement import *
from unit.models.codecs import DtoDecoder


class StatementResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "statements"

    def get(self, params: GetStatementParams) -> Union[UnitResponse[str], UnitError]:
        parameters = {"language": params.language}
        if params.customer_id:
            parameters["filter[customerId]"] = params.customer_id

        response = super().get(f"{self.resource}/{params.statement_id}/{params.output_type}", parameters)
        if response.status_code == 200:
            return UnitResponse[str](response.text, None)
        else:
            return UnitError.from_json_api(response.json())

    def get_bank_verification(self, account_id: str, include_proof_of_funds: Optional[bool] = False) -> Union[UnitResponse[str], UnitError]:
        response = super().get(f"{self.resource}/{account_id}/bank/pdf",
                               {"includeProofOfFunds": include_proof_of_funds})
        if response.status_code == 200:
            return UnitResponse[str](response.text, None)
        else:
            return UnitError.from_json_api(response.json())

    def list(self, offset: int = 0, limit: int = 100) -> Union[UnitResponse[List[StatementDTO]], UnitError]:
        response = super().get(self.resource, {"page[limit]": limit, "page[offset]": offset})
        if response.status_code == 200:
            data = response.json().get("data")
            return UnitResponse[StatementDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())




from unit.api.base_resource import BaseResource
from unit.models.counterparty import *
from unit.models.codecs import DtoDecoder


class CounterpartyResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "counterparties"

    def create(self, request: Union[CreateCounterpartyRequest, CreateCounterpartyWithTokenRequest]) -> Union[UnitResponse[CounterpartyDTO], UnitError]:
        payload = request.to_json_api()
        response = super().post(self.resource, payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[CounterpartyDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def update(self, request: PatchCounterpartyRequest) -> Union[UnitResponse[CounterpartyDTO], UnitError]:
        payload = request.to_json_api()
        response = super().patch(f"{self.resource}/{request.counterparty_id}", payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[CounterpartyDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def delete(self, counterparty_id: str) -> Union[UnitResponse, UnitError]:
        response = super().delete(f"{self.resource}/{counterparty_id}")
        if super().is_20x(response.status_code):
            return UnitResponse([], None)
        else:
            return UnitError.from_json_api(response.json())

    def get(self, counterparty_id: str) -> Union[UnitResponse[CounterpartyDTO], UnitError]:
        response = super().get(f"{self.resource}/{counterparty_id}")
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[CounterpartyDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def list(self, offset: int = 0, limit: int = 100) -> Union[UnitResponse[List[CounterpartyDTO]], UnitError]:
        response = super().get(self.resource, {"page[limit]": limit, "page[offset]": offset})
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[CounterpartyDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())



import json
from typing import Optional
from unit.models import *


class AccountEndOfDayDTO(object):
    def __init__(self, id: str, date: str, balance: int, hold: int, available: int,
                 relationships: Optional[Dict[str, Relationship]]):
        self.id = id
        self.type = "accountEndOfDay"
        self.attributes = {"date": date, "balance": balance, "hold": hold, "available": available}
        self.relationships = relationships

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return AccountEndOfDayDTO(_id, attributes["date"], attributes["balance"], attributes["hold"],
                                  attributes["available"], relationships)


class AccountEndOfDayListParams(object):
    def __init__(self, limit: int = 100, offset: int = 0, account_id: str = "", customer_id: str = "", since: str = "",
                 until: str = ""):
        self.limit = limit
        self.offset = offset
        self.account_id = account_id
        self.customer_id = customer_id
        self.since = since
        self.until = until


from unit.api.base_resource import BaseResource
from unit.models.account_end_of_day import *
from unit.models.codecs import DtoDecoder


class AccountEndOfDayResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "account-end-of-day"

    def list(self, params: AccountEndOfDayListParams = AccountEndOfDayListParams()) -> Union[UnitResponse[List[AccountEndOfDayDTO]], UnitError]:
        parameters = {"page[limit]": params.limit, "page[offset]": params.offset}

        if params.account_id != "":
            parameters |= {"filter[accountId]": params.account_id}

        if params.customer_id != "":
            parameters |= {"filter[customerId]": params.customer_id}

        if params.since != "":
            parameters |= {"filter[since]": params.since}

        if params.until != "":
            parameters |= {"filter[until]": params.until}

        response = super().get(self.resource, parameters)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[AccountEndOfDayDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())




from unit.api.base_resource import BaseResource
from unit.models.event import *
from unit.models.codecs import DtoDecoder


class EventResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "events"

    def get(self, event_id: str) -> Union[UnitResponse[EventDTO], UnitError]:
        response = super().get(f"{self.resource}/{event_id}")
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[EventDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def list(self, offset: int = 0, limit: int = 100) -> Union[UnitResponse[List[EventDTO]], UnitError]:
        response = super().get(self.resource, {"page[limit]": limit, "page[offset]": offset})
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[EventDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def fire(self, event_id: str) -> Union[UnitResponse, UnitError]:
        response = super().post(f"{self.resource}/{event_id}")
        if super().is_20x(response.status_code):
            return UnitResponse([], None)
        else:
            return UnitError.from_json_api(response.json())



import json
from typing import Optional
from unit.models import *


class BillerDTO(object):
    def __init__(self, id: str, name: int, category: str):
        self.id = id
        self.type = "biller"
        self.attributes = {"name": name, "category": category}

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return BillerDTO(_id, attributes["name"], attributes["category"])


class GetBillersParams(object):
    def __init__(self, name: str, page: Optional[int] = None):
        self.name = name
        self.page = page




import json
from typing import Optional
from unit.models import *


class FeeDTO(object):
    def __init__(self, id: str, amount: int, description: str, tags: Optional[Dict[str, str]],
                 relationships: Optional[Dict[str, Relationship]]):
        self.id = id
        self.type = "fee"
        self.attributes = {"amount": amount, "description": description, "tags": tags}
        self.relationships = relationships

    @staticmethod
    def from_json_api(_id, _type, attributes, relationships):
        return FeeDTO(_id, attributes["amount"], attributes["description"], attributes.get("tags"), relationships)


class CreateFeeRequest(object):
    def __init__(self, amount: int, description: str, relationships: Optional[Dict[str, Relationship]],
                 tags: Optional[Dict[str, str]] = None, idempotency_key: Optional[str] = None):
        self.amount = amount
        self.description = description
        self.tags = tags
        self.idempotency_key = idempotency_key
        self.relationships = relationships

    def to_json_api(self) -> Dict:
        payload = {
            "data": {
                "type": "fee",
                "attributes": {
                    "amount": self.amount,
                    "description": self.description
                },
                "relationships": self.relationships
            }
        }

        if self.idempotency_key:
            payload["data"]["attributes"]["idempotencyKey"] = self.tags

        if self.tags:
            payload["data"]["attributes"]["tags"] = self.tags

        return payload

    def __repr__(self):
        json.dumps(self.to_json_api())




from unit.api.base_resource import BaseResource
from unit.models.webhook import *
from unit.models.codecs import DtoDecoder
import hmac
from hashlib import sha1
import base64


class WebhookResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "webhooks"

    def create(self, request: CreateWebhookRequest) -> Union[UnitResponse[WebhookDTO], UnitError]:
        payload = request.to_json_api()
        response = super().post(self.resource, payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[WebhookDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def get(self, webhook_id: str) -> Union[UnitResponse[WebhookDTO], UnitError]:
        response = super().get(f"{self.resource}/{webhook_id}")
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[WebhookDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def list(self, offset: int = 0, limit: int = 100) -> Union[UnitResponse[List[WebhookDTO]], UnitError]:
        response = super().get(self.resource, {"page[limit]": limit, "page[offset]": offset})
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[WebhookDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def update(self, request: PatchWebhookRequest) -> Union[UnitResponse[WebhookDTO], UnitError]:
        payload = request.to_json_api()
        response = super().patch(f"{self.resource}/{request.webhook_id}", payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[WebhookDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def enable(self, webhook_id: str) -> Union[UnitResponse[WebhookDTO], UnitError]:
        response = super().post(f"{self.resource}/{webhook_id}/enable")
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[WebhookDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def disable(self, webhook_id: str) -> Union[UnitResponse[WebhookDTO], UnitError]:
        response = super().post(f"{self.resource}/{webhook_id}/disable")
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[WebhookDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def verify(self, signature: str, secret: str, payload):
        mac = hmac.new(
            secret.encode(),
            msg=json.dumps(payload, separators=(',', ':'), ensure_ascii=False).encode('utf-8'),
            digestmod=sha1,
        )
        res = base64.encodebytes(mac.digest()).decode().rstrip('\n')
        return res == signature



from unit.api.application_resource import ApplicationResource
from unit.api.customer_resource import CustomerResource
from unit.api.account_resource import AccountResource
from unit.api.card_resource import CardResource
from unit.api.transaction_resource import TransactionResource
from unit.api.payment_resource import PaymentResource
from unit.api.statement_resource import StatementResource
from unit.api.customerToken_resource import CustomerTokenResource
from unit.api.counterparty_resource import CounterpartyResource
from unit.api.returnAch_resource import ReturnAchResource
from unit.api.applicationForm_resource import ApplicationFormResource
from unit.api.fee_resource import FeeResource
from unit.api.event_resource import EventResource
from unit.api.webhook_resource import WebhookResource
from unit.api.institution_resource import InstitutionResource
from unit.api.atmLocation_resource import AtmLocationResource
from unit.api.bill_pay_resource import BillPayResource
from unit.api.api_token_resource import APITokenResource
from unit.api.authorization_resource import AuthorizationResource
from unit.api.authorization_request_resource import AuthorizationRequestResource
from unit.api.account_end_of_day_resource import AccountEndOfDayResource

__all__ = ["api", "models", "utils"]


class Unit(object):
    def __init__(self, api_url, token):
        self.applications = ApplicationResource(api_url, token)
        self.customers = CustomerResource(api_url, token)
        self.accounts = AccountResource(api_url, token)
        self.cards = CardResource(api_url, token)
        self.transactions = TransactionResource(api_url, token)
        self.payments = PaymentResource(api_url, token)
        self.statements = StatementResource(api_url, token)
        self.customerTokens = CustomerTokenResource(api_url, token)
        self.counterparty = CounterpartyResource(api_url, token)
        self.returnAch = ReturnAchResource(api_url, token)
        self.applicationForms = ApplicationFormResource(api_url, token)
        self.fees = FeeResource(api_url, token)
        self.events = EventResource(api_url, token)
        self.webhooks = WebhookResource(api_url, token)
        self.institutions = InstitutionResource(api_url, token)
        self.atmLocations = AtmLocationResource(api_url, token)
        self.billPays = BillPayResource(api_url, token)
        self.api_tokens = APITokenResource(api_url, token)
        self.authorizations = AuthorizationResource(api_url, token)
        self.authorization_requests = AuthorizationRequestResource(api_url, token)
        self.account_end_of_day = AccountEndOfDayResource(api_url, token)



from unit.api.base_resource import BaseResource
from unit.models.api_token import *
from unit.models.codecs import DtoDecoder


class APITokenResource(BaseResource):
    def __init__(self, api_url, token):
        super().__init__(api_url, token)
        self.resource = "users"

    def create(self, request: CreateAPITokenRequest) -> Union[UnitResponse[APITokenDTO], UnitError]:
        payload = request.to_json_api()
        response = super().post(f"{self.resource}/{request.user_id}/api-tokens", payload)
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[APITokenDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def list(self, user_id: str) -> Union[UnitResponse[List[APITokenDTO]], UnitError]:
        response = super().get(f"{self.resource}/{user_id}/api-tokens")
        if super().is_20x(response.status_code):
            data = response.json().get("data")
            return UnitResponse[APITokenDTO](DtoDecoder.decode(data), None)
        else:
            return UnitError.from_json_api(response.json())

    def revoke(self, user_id: str, token_id: str) -> Union[UnitResponse, UnitError]:
        response = super().delete(f"{self.resource}/{user_id}/api-tokens/{token_id}")
        if super().is_20x(response.status_code):
            return UnitResponse([], None)
        else:
            return UnitError.from_json_api(response.json())

